/*
 *  Copyright 2007 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.22 01-10-2007 (ndk-b22)
 */
//
// llpacket.h
//
// Useful equates for a packet driver
//

//
// Pre-Pad Packet Data Offset
//
// The TCP/IP stack library requires that every packet device
// include enough L2 header room for all supported headers. In
// order to support PPPoE, this requires a 22 byte L2 header.
// Thus, since standard Ethernet is only 14 bytes, we must add
// on an additional 8 byte offset, or PPPoE can not function
// with our driver.
//
#define PKT_PREPAD          8

// Multicast Address List Size
#define PKT_MAX_MCAST       16

// Packet Queues
#ifndef _INCLUDE_NIMU_CODE
extern PBMQ    PBMQ_rx;         // Rx queue (one for all PKT devices)
#endif

//
// Packet device information
//
typedef struct _pdinfo {
    uint            PhysIdx;    // Physical index of this device (0 to n-1)
    HANDLE          hEther;     // Handle to logical driver
    STKEVENT_Handle hEvent;
    UINT8           bMacAddr[6];// MAC Address
    uint            Filter;     // Current RX filter
    uint            MCastCnt;   // Current MCast Address Countr
    UINT8           bMCast[6*PKT_MAX_MCAST];
    uint            TxFree;     // Transmitter "free" flag
    PBMQ            PBMQ_tx;    // Tx queue (one for each PKT device)
#ifdef _INCLUDE_NIMU_CODE
    PBMQ    PBMQ_rx;         // Rx queue (one for all PKT devices)
#endif
} PDINFO;

//
// Functions Provided by Hw specific code
//
extern uint HwPktInit();
extern void HwPktShutdown();
extern uint HwPktOpen( PDINFO *pi );
extern void HwPktClose( PDINFO *pi );
extern void HwPktSetRx( PDINFO *pi );
extern void HwPktTxNext( PDINFO *pi );
extern void _HwPktPoll( PDINFO *pi, uint fTimerTick );
